#!/bin/sh
# this sets decimal point to dot. Otherwise in other locale settings (for example Slovenian), decimal point is comma and 
# awk does not work correctly (does not recognize numbers correctly)
export LANG=en_US.UTF-8

# default column is 2 ( first column is 1 ) 
COL=2
LOOP="1"

# check all input parameters
if [ "$1" == "-c" ]; then
	# sort column is defined
	COL=$2
	shift 2;
	
fi
	
if [ "$1" == "-seq" ]; then
	# next parameters are: input file, start, step, stop steps for input files;
	# input file should have C format for printing step number 
	FILE_STEP=1
	INPFILE=$2
	START=$3
	STEP=$4
	STOP=$5
	shift 4;

	FILE=""

	for i in `seq $START $STEP $STOP`; do
		FN=`printf "$INPFILE" $i`
		FILE=`echo $FILE $FN`
	done

	LOOP="0"

else
	# input file was define as shell expansion (*)
	if [ -z "$1" ]; then
		# no input parameters; display usage
		echo "Usage: $0 [-c col_num] [-seq filename min step max] [filename(s)]"
		echo ""
		echo "  -c col_num: in which colun to check for min and max values; use 0 for first column"
		echo "  -seq filename: define filename, with C printf format for current step number."
		echo "                 example: result_%05d.dat -- %05d is replaced with current step number."
		echo "       min step max: start, step and end values for simulation step number"
		echo "  filename(s): filenames to check - including * and ?. Use quotes to prevent shell expansion."
		echo "               example: \"*.dat\" -- to check all files in current directory that end with .dat ."
		echo ""
		echo "Lines that start with hash (#) are treated as comments and are skipped."
		exit 1
	fi

	FILE=$1
fi

# define empty values for limits so limits are first defined in awk (see script below)
VMIN=
VMAX=

for i in $FILE; do
	# parse through all given files...
	set -- `cat $i | awk -v C=$COL -v MIN=$VMIN -v MAX=$VMAX '
		BEGIN { vmin=MIN ; vmax=MAX }
		{ 
			if( ! /^#/ ) {
				if( vmin == "" && vmax == "" ) {
					# first line in first file;
					vmin=$C;
					vmax=$C;
				}
			
				if( $C < vmin ) { vmin=$C; };
				if( $C > vmax ) { vmax=$C; };
			}
		}
		END { print vmin " " vmax; }'`

	# output of command is stored into $1 and $2 variables; read them; they are used for next file
	VMIN=$1
	VMAX=$2 
done

# last VMIN and VMAX values are limits for all files
echo $VMIN":"$VMAX 

# done!
