#!/bin/sh
# this sets decimal point to dot. Otherwise in other locale settings (for example Slovenian), decimal point is comma and 
# awk does not work correctly (does not recognize numbers correctly)
export LANG=en_US.UTF-8
TMPFILE="tmp.png"

# columns to display
COL_X=3
COL_Y=6

#
# display script usage
#
usage() {
  echo "Usage:"
  echo "$0 <final GIF file> <file pattern file_%08d.dat> <start step no.> <step increment> <end. step no.>"
  echo
  exit 
}

if [ -z "$5" ]; then
  usage
  exit 
fi

# delete old images (not to mess with new (generated) ones)
echo "Removing old images..."
rm -f *.png *.jpg

# get parameters
OUTFILE=$1
FILE_PATT=$2
START=$3
STEP=$4
END=$5
shift 5
FILES=""

# relative directory for running find_max....
REL_DIR=`dirname $0`

echo
echo "Finding min and max values for given variable..."

# get Y limits so al images are displayed in same limits
Y_LIMITS=`${REL_DIR}/find_min_max_v2.sh -c ${COL_Y} -seq $FILE_PATT $START $STEP $END`
# limits for X-axis; in format "min:max"
X_LIMITS=`${REL_DIR}/find_min_max_v2.sh -c ${COL_X} -seq $FILE_PATT $START $STEP $END`

echo "Y-axis limits: $Y_LIMITS"
echo "X-axis limits: $X_LIMITS"

echo
echo "Generating images (displaying current step)..."

for i in `seq $START $STEP $END`; do
	echo -n " $i"
	N=`printf "%08d" $i`
	COMM="set terminal png medium size 800,600 ; set output '$TMPFILE' ; set grid xtics ytics ; plot [${X_LIMITS}] [${Y_LIMITS}] "

	FILE=`printf $FILE_PATT $i`
	COMM=`echo $COMM "'${FILE}'" using ${COL_X}:${COL_Y}`

	# now generate images; 
	#echo $COMM
	echo $COMM | gnuplot
	convert tmp.png -type truecolor movie_${N}.gif
done

echo
echo "making animated gif..."

convert -delay 20 movie_*.gif ${OUTFILE}
rm -f $TMPFILE
echo "All done!"

