#!/bin/sh
# this sets decimal point to dot. Otherwise in other locale settings (for example Slovenian), decimal point is comma and 
# awk does not work correctly (does not recognize numbers correctly)
export LANG=en_US.UTF-8

if [ -z "$1" ]; then
  #usage
  echo "Usage:"
  echo "$0 <start step no.> <step increment> <end. step no.>"
  echo
  exit 
fi

START=$1
STEP=$2
END=$3
REL_DIR=`dirname $0`
OUTMOVIE=X.gif

# some settings, used in few places in script
PH_COL_X=3
PH_COL_V=6
POT_COL_X=2
POT_COL_PHI=3

rm -f *.gif

echo
echo "Finding min and max values for phase and potential..."

PH_LIMITS=`${REL_DIR}/find_min_max_v2.sh -c ${PH_COL_V} -seq "phase-space_%07d.dat" $START $STEP $END`
POT_LIMITS=`${REL_DIR}/find_min_max_v2.sh -c ${POT_COL_PHI} -seq "potential_%07d.dat" $START $STEP $END`
# limits for displaying phase-space diagram Y axis values; in format "min:max"
#PH_LIMITS='-3.5:2.0'
# limits for displaying potential; in format "min:max"
#POT_LIMITS='-1:0.5'
# limits for X-axis; in format "min:max"

# find limits for X axis
#N=`printf "%07d" $START`
#X_LIMITS=`${REL_DIR}/find_min_max_v2.sh -c ${PH_COL_X} "phase-space_${N}.dat"`
X_LIMITS="0:0.0612"

#X_LIMITS="0:0.06"
# values for ticks on X axis; in format "min, step, max"
#X_TICKS="0, 0.005, 0.06"

echo "X-axis limits: $X_LIMITS"
echo "PH limits: $PH_LIMITS"
echo "POT limits: $POT_LIMITS"

echo
echo "Generating images (displaying current step)..."

for i in `seq $START $STEP $END`; do
	echo -n " $i"
	N=`printf "%07d" $i`
	
	# now generate images; 
	# images have names: results_N_NS_vx.png and results_N_NS_fx.png
	echo "set terminal png medium size 640, 300 ; set output 'ph-space.png' ; set grid xtics ytics ; plot [${X_LIMITS}] [${PH_LIMITS}] 'phase-space_${N}.dat' using ${PH_COL_X}:${PH_COL_V} title 'Velocity vs. Position - step ${N}'" | gnuplot
	convert ph-space.png -type truecolor phase-space.jpg
	
	echo "set terminal png medium size 640, 300 ; set output 'potential.png' ; set grid xtics ytics ; plot [${X_LIMITS}] [${POT_LIMITS}] 'potential_${N}.dat' using ${POT_COL_X}:${POT_COL_PHI} title 'Potential vs. Position - step ${N}'" | gnuplot
	convert potential.png -type truecolor potential.jpg
	
	# join both images into one that is later used for making movie
	montage -tile 1x2 -mode Concatenate phase-space.jpg potential.jpg movie_${N}.gif
done
echo
echo "making movie..."

convert -delay 20 movie_*.gif ${OUTMOVIE}
echo "All done!"

