/*
 * Compute.cpp
 *
 *  Created on: Aug 11, 2011
 *      Author: janez
 */

#include "Compute_green.h"

#include "iostream"
using namespace std;

//#define DISPLAY_POTENTIAL_DEBUG


Compute_green::Compute_green() {
	A = B = 0;

}

Compute_green::~Compute_green() {

}


/**
 * Compute forces on particles for given particle list.
 */
void Compute_green::forces( ParticleList& in_pl, Tree& in_tree, SpatialRegion& in_region ) {
	long int i;
	Particle* pt;
	Vector3 F;


	// store input parameters for use in all internal functions...
	particlelist = &in_pl;
	tree = &in_tree;
	region = &in_region;

	for( i=0 ; i < particlelist->size() ; i++ ) {
		// calculate force for all particles
		pt = particlelist->getParticle(i);
		// calculate F...
		F = forceOnParticle( pt );
		// store calculated force back to particle
		pt->set_f( F );
	}
}


/**
 * Compute forces on particles for given particle list.
 */
Vector3 Compute_green::force( ParticleList& in_pl, SpatialRegion& in_region, Particle pt ) {
	Tree tmp_tree;

	// store input parameters for use in all internal functions...
	particlelist = &in_pl;
	tree = &tmp_tree;
	region = &in_region;

	// build tree for computing force
	tmp_tree.init();
	tmp_tree.build( in_pl, in_region );
	// compute force on particle and return in
	return forceOnParticle( &pt );
}


/**
 * Compute force on given particle - intern method, called by other methods.
 */
Vector3 Compute_green::forceOnParticle( Particle *pt ) {
	double Fx = 0, Ei;
	Tree tmp_tree;
	Vector3 F, pos;


	if( tree->get_root() != NULL ) {
		// tree is defined
		if( tree->get_treecode_disable() == true ) {
			// use direct simulation
			pos = pt->get_r();
			// calculate E...
			Ei = 1 / ( 2 * region->get_area() * EPS0 );
			Ei *= compute_Ec_directSum( particlelist, tree->get_root(), &pos );
			// ...and force...
			Fx = + pt->get_q() * Ei;

		} else {
			// OK, use treecode
			calculate_constants_A_B();
			pos = pt->get_r();
			// calculate E...
			Ei = 1 / ( 2 * region->get_area() * EPS0 );
			Ei *= compute_Ec_tc( particlelist, tree->get_root(), &pos );
			// ...and force...
			Fx = + pt->get_q() * ( Ei + this->A );
		}
	}

	// store force and return it
	F.set_e1(Fx);

	return F;
}

/**
 * Calculate potential on grid, defined for given region.
 */
vector<double> Compute_green::potential( ParticleList& in_pl, Tree& in_tree, SpatialRegion& in_region ) {
	vector<double> pot;
	unsigned int i, max_i;
	double x;


	// initialize potential array
	pot.resize( in_region.get_grid().get_n(0) );

	for( i=0, max_i=in_region.get_grid().get_n(0) ; i < max_i ; i++ ) {
		// initialize potential array; fill with zeros
		pot[i] = 0.;
	}

	if( in_tree.get_root() != NULL ) {
		// tree is defined
		// store input parameters for use in all internal functions...
		particlelist = &in_pl;
		tree = &in_tree;
		region = &in_region;
		// calculate factors
		calculate_constants_A_B();

		for( i=0, max_i=pot.size() ; i < max_i ; i++ ) {
			// do potential computation for all positions on grid and for all particle species
			x = region->get_grid().get_x(0,i);
			// add potential value to final array
			pot[i] = - potential_p( x ) - potential_h( x );
		}
	}

	return pot;
}


/**
 * Compute particular solution for potential in given position and for given particles and tree
 *
 */
double Compute_green::potential_p( double x ) {
	double pot=0;


	if( tree->get_root() != NULL ) {
		// tree is defined
		pot = 1 / ( 2 * region->get_area() * EPS0 );
		pot *= compute_Phic_tc( particlelist, tree->get_root(), x );
	}

#ifdef DISPLAY_POTENTIAL_DEBUG
	cout << "pot_p(" << x << ") = " << pot << endl;
#endif

	return pot;
}



/**
 * Calculate constants A and B that are used later for calculating other values.
 */
void Compute_green::calculate_constants_A_B() {
	double x0 = region->get_grid_x_min();
	double x1 = region->get_grid_x_max();
	double pot_p0 = potential_p( x0 );
	double pot_p1 = potential_p( x1 );
	double V0 = region->get_V0();
	double V1 = region->get_V1();

	// calculate constants
	this->A = 1/(x1 - x0) * (V1 - V0 + pot_p0 - pot_p1);
	this->B = x1/(x1 - x0) * (V0 - pot_p0) - x0/(x1 - x0) * (V1 - pot_p1);
}

#if 0
/**
 * Compute electric field between particle (given by index) caused by particles in current tree.
 */
double Compute_green::compute_Ec_tc( ParticleList *pl, struct tc_node *node, long int pt_index ) {
	double r, l;
	double Ec=0;

/*
	if( node->Qc == 0 ) {
		// compute charge and middle point for given node in tree
		node->Qc = gf_computeQc(parts, tree);
		node->Xc = gf_computeXc(tree);
	}
*/
	if( node == NULL ) {
		// node not defined; return
		cout << "compute_Ec_tc: tree node is empty! particle index: " << pt_index << endl;
		return Ec;
	}

	/* calculate distance between current point and center of cluster */
	l = pl->getParticle(pt_index)->get_r().e1() - node->Xc;
	/* calculate radius of tree node */
	r = fabs(node->de - node->ds)/2;

	if( r < fabs(l) * tree->get_p2c() ) {
		/* compute using particle-to-cluster interaction: particle is "far" from cluster (node) */
		Ec = compute_Ec_single( node->Qc, l );

	} else {
		if( node->l1 == NULL && node->l2 == NULL ) {
			/* this is last tree node and we have to compute force with direct sum */
			Ec = compute_Ec_directSum( pl, node, pt_index );

		} else {
			Ec = 0;

			if( node->l1 != NULL ) {
				/* compute for first leaf of a node */
				Ec += compute_Ec_tc( pl, node->l1, pt_index );
			}

			if( node->l2 != NULL ) {
				/* compute for second leaf of a node */
				Ec += compute_Ec_tc( pl, node->l2, pt_index );
			}
		}
	}

	return Ec;
}
#endif

/**
 * Compute E field to given position with treecode.
 */
double Compute_green::compute_Ec_tc( ParticleList *pl, TreeNode *node, Vector3 *pt_pos ) {
	double r, l;
	double Ec=0;

/*
	if( node->Qc == 0 ) {
		// compute charge and middle point for given node in tree
		node->Qc = gf_computeQc(parts, tree);
		node->Xc = gf_computeXc(tree);
	}
*/

	if( node == NULL ) {
		// node not defined; return
		cout << "compute_Ec_tc: tree node is empty! Position: (" << pt_pos->e1() << ", " << pt_pos->e2() << ", " << pt_pos->e3() << ")" << endl;
		return Ec;
	}

	/* calculate distance between current point and center of cluster */
	l = node->getDistanceFromCenter(pt_pos->e1());
	/* calculate radius of tree node */
	r = node->getRadius();

	if( r < fabs(l) * tree->get_p2c() ) {
		/* compute using particle-to-cluster interaction: particle is "far" from cluster (node) */
		Ec = compute_Ec_single( node->getQc(), l );

	} else {
		if( node->getLeaf1() == NULL && node->getLeaf2() == NULL ) {
			/* this is last tree node and we have to compute the force with direct sum */
			Ec = compute_Ec_directSum( pl, node, pt_pos );

		} else {
			Ec = 0;

			if( node->getLeaf1() != NULL ) {
				/* compute for first leaf of a node */
				Ec += compute_Ec_tc( pl, node->getLeaf1(), pt_pos);
			}

			if( node->getLeaf2() != NULL ) {
				/* compute for second leaf of a node */
				Ec += compute_Ec_tc( pl, node->getLeaf2(), pt_pos);
			}
		}
	}

	return Ec;
}


#if 0
/**
 * Compute E field with direct sum of forces between particle (given by index) and particles in given node.
 */
double Compute_green::compute_Ec_directSum( ParticleList *pl, struct tc_node *node, long int pt_index ){
	Particle *pt;

	pt = pl->getParticle(pt_index)->get_r();

	return compute_Ec_directSum( pl, node, &pt );
}
#endif

/**
 * Compute E field with direct sum of forces between particle (given by index) and particles in given node.
 */
double Compute_green::compute_Ec_directSum( ParticleList *pl, TreeNode *node, Vector3 *pt_pos ){
	long int index;
	double Ec=0, pt_x;
	Particle *cur_pt;

	// position of particle to which we calculate forces of other particles
	pt_x = pt_pos->e1();

	for( unsigned int i=0 ; i < node->getNumberOfParticles() ; i++) {
		/* use all particles in current tree node */
		index = node->getParticleIndex(i);
		cur_pt = pl->getParticle(index);

		if( !IS_ZERO(pt_x - cur_pt->get_r().e1()) ) {
			/* not on "calculated" index so we can calculate force between current particles */
			Ec += compute_Ec_single(cur_pt->get_q(), pt_x - cur_pt->get_r().e1() );
		}
	}

	return Ec;
}


/**
 * Compute potential in given position caused by particles in current tree.
 */
double Compute_green::compute_Phic_tc( ParticleList *pl, TreeNode *node, double x ) {
	double r, l;
	double Phi_c=0;

/*
	if( node->Qc == 0 ) {
		// compute charge and middle point for given node in tree
		node->Qc = gf_computeQc(parts, tree);
		node->Xc = gf_computeXc(tree);
	}
*/

	if( node == NULL ) {
		// node not defined; return
		cout << "compute_Phic_tc: tree node is empty! x=" << x << endl;
		return Phi_c;
	}

	/* calculate distance between current point and center of cluster */
	l = node->getDistanceFromCenter(x);
	/* calculate radius of tree node */
	r = node->getRadius();

	if( r < fabs(l) * tree->get_p2c() ) {
		/* compute using particle-to-cluster interaction: particle is "far" from cluster (node) */
		Phi_c = compute_Phic_single( node->getQc(), l );

	} else {
		if( node->getLeaf1() == NULL && node->getLeaf2() == NULL ) {
			/* this is last tree node and we have to compute potential with direct sum */
			Phi_c = compute_Phic_directSum( pl, node, x );

		} else {
			Phi_c = 0;

			if( node->getLeaf1() != NULL ) {
				/* compute for first leaf of a node */
				Phi_c += compute_Phic_tc( pl, node->getLeaf1(), x );
			}

			if( node->getLeaf2() != NULL ) {
				/* compute for second leaf of a node */
				Phi_c += compute_Phic_tc( pl, node->getLeaf2(), x );
			}
		}
	}

	return Phi_c;
}


/**
 * Compute potential with direct sum of forces between given position and particles in given node.
 */
double Compute_green::compute_Phic_directSum( ParticleList *pl, TreeNode *node, double x ){
	long int index;
	double Phi_c=0, diff;
	Particle *cur_pt;

	// position to which we calculate forces due to other particles is defined in "x"

	for( unsigned int i=0 ; i < node->getNumberOfParticles() ; i++) {
		/* use all particles in current tree node */
		index = node->getParticleIndex(i);
		cur_pt = pl->getParticle(index);
		diff = x - cur_pt->get_r().e1();

		if( !IS_ZERO(diff) ) {
			/* not on "calculated" position so we can calculate force between given position and current particle */
			Phi_c += compute_Phic_single( cur_pt->get_q(), diff );
		}
	}

	return Phi_c;
}
