/*
 * Dump.cpp
 *
 *  Created on: Aug 10, 2011
 *      Author: janez
 */

#include "Dump.h"

#include <iostream>
#include <fstream>
using namespace std;


Dump::Dump() {
	filename = "";

}

Dump::~Dump() {


}


/**
 * Load dump data from given file.
 */
bool Dump::load(std::string inFilename) {

	return true;
}


/**
 * Save current values in program to given dump file
 */
bool Dump::save(std::string inFilename) {

	return true;
}

/**
 * Display given particle list to stdout.
 */
void Dump::display( ParticleList& pl, std::vector<Species>&sl, std::string comment ) {
	unsigned int i, j, k;
	Particle *pt;


	cout << endl;
	cout << "Particles display" << endl;
	cout << "=================" << endl;
	cout << "Comment: " << comment << endl;
	cout << "Total number of particles: " << pl.size() << endl;

	for( i=0 ; i < sl.size() ; i++ ) {
		// display species and particles
		cout << sl[i].dump2string();
		cout << "no. i sp. r(x y z)  v(x y z)  f(x y z) q m" << endl;

		for( j=0, k=0 ; j < pl.size() ; j++ ) {
			// display particle info
			pt = pl.getParticle(j);

			if( (unsigned) pt->get_sp_index() == i ) {
				// display only particles of current species
				cout << j << " " << k << " " << pt->get_sp_index() << " (" << pt->get_str_r() << ") (" << pt->get_str_v();
				cout << ") (" << pt->get_str_f() << ")" << " " << pt->get_q() << " " << pt->get_m() << endl;
				k++ ;
			}
		}
	}

	cout << "--- end of particle display ---" << endl;
}


/**
 * Display given potential list (with spatial region for coordinates) to stdout.
 */
void Dump::display( std::vector<double>& pot, SpatialRegion& region, std::string comment ) {
	unsigned int i;

	if( pot.size() != (unsigned) region.get_grid().get_nx() ) {
		// not the same size;
		cout << endl << "Sizes of potential values (" << pot.size() << ") and ";
		cout << "grid cells (" << region.get_grid().get_nx() << ") are not the same. Exiting." << endl;
		return;
	}

	cout << "Comment: " << comment << endl;
	cout << "Number of potential points: " << pot.size() << endl;
	cout << " i.  x    potential " << endl;

	for( i=0 ; i < pot.size() ; i++ ) {
		// display potential results
		cout << " " << i << ". " << region.get_grid().get_x(0,i) << " " << pot[i] << endl;
	}

	cout << "--end--" << endl;
}


/**
 * Display given particle list to file, given by "toFile".
 */
void Dump::toFile( std::string toFile, ParticleList& pl, std::vector<Species>&sl ) {
	unsigned int i;
	Particle *pt;
	fstream fout;


	// open file
	fout.open( toFile.c_str(), fstream::out );

	if( fout.is_open() ) {
		// OK, file read to write into
		// display header row (comment): number of species, names of all columns
		fout << "# particle data; number of particles: " << pl.size() << ", row data: i x vx vy vz" << endl;

		for( i=0 ; i < pl.size() ; i++ ) {
			// display data for all species
			pt = pl.getParticle(i);
			fout << i << " " << pt->get_str_rx() << " " << pt->get_str_v() << endl;
		}

		fout.close();
	} else {
		cout << "Error: could not open file " << toFile << " for writing." << endl;
	}
}



/**
 * Display given potential list (with spatial region for coordinates) to file with name "toFile".
 */
void Dump::toFile( std::string toFile, std::vector<double>& pot, SpatialRegion& region ) {
	unsigned int i;
	double x;
	fstream fout;


	if( pot.size() != (unsigned) region.get_grid().get_nx() ) {
		// not the same size;
		cout << endl << "Sizes of potential values (" << pot.size() << ") and ";
		cout << "grid cells (" << region.get_grid().get_nx() << ") are not the same. Exiting." << endl;
		return;
	}

// cout << endl << "pot(x=0) = " << pot[0] << ", pot(x=L)=" << pot[pot.size()-1] << endl;

	// open file
	fout.open( toFile.c_str(), fstream::out );

	if( fout.is_open() ) {
		// OK, file read to write into
		// display header row (comment): number of species, names of all columns
		fout << "# potential. row data: x potential" << endl;

		for( i=0 ; i < pot.size() ; i++ ) {
			// display potential results
//			fout << i << " " << region.get_grid().get_x(0,i) << " " << pot[i] << endl;
			x = region.get_grid().get_x(0,i);
			fout << i << " " << x << " " << pot[i] << endl;
		}

		fout.close();
	} else {
		cout << "Error: could not open file " << toFile << " for writing." << endl;
	}
}

