/*
 * RandomGenerator.h
 *
 *  Created on: 2012-01-14
 *      Author: janez
 *
 * For MT19937 generator:
 * A C-program for MT19937-64 (2004/9/29 version).
 * Coded by Takuji Nishimura and Makoto Matsumoto.
 *
 * This is a 64-bit version of Mersenne Twister pseudorandom number
 * generator.
 *
 * Before using, initialize the state by using init_genrand64(seed)
 * or init_by_array64(init_key, key_length).
 *
 * Copyright (C) 2004, Makoto Matsumoto and Takuji Nishimura,
 * All rights reserved.
 *
 *
 */

#ifndef RANDOMGENERATOR_H_
#define RANDOMGENERATOR_H_

/* Period parameters */
#define NN 312
#define MM 156
#define MATRIX_A 0xB5026F5AA96619E9ULL
#define UM 0xFFFFFFFF80000000ULL /* Most significant 33 bits */
#define LM 0x7FFFFFFFULL /* Least significant 31 bits */


class RandomGenerator {
	int method;

	// for system random function
	//unsigned int rand_state;



	// for MT19937 generator
	long long mt[NN];		/* The array for the state vector */
	int mti;
	unsigned long long mt_init[4], mt_length;
	/* initializes mt[NN] with a seed */
	void init_genrand64(unsigned long long seed);
	/* initialize by an array with array-length */
	/* init_key is the array for initializing keys */
	/* key_length is its length */
	void init_by_array64(unsigned long long init_key[], unsigned long long key_length);
	/* generates a random number on [0, 2^64-1]-interval */
	unsigned long long genrand64_int64(void);
	/* generates a random number on [0, 2^63-1]-interval */
	long long genrand64_int63(void);
	/* generates a random number on [0,1]-real-interval */
	double genrand64_real1(void);
	/* generates a random number on [0,1)-real-interval */
	double genrand64_real2(void);
	/* generates a random number on (0,1)-real-interval */
	double genrand64_real3(void);

public:
	static const int RANDOM_GENERATOR_SYSTEM = 0;
	static const int RANDOM_GENERATOR_MT19937 = 1;
	RandomGenerator();
	virtual ~RandomGenerator();
	void Init();
	void Init(int in_method);
	void set_generator( int generator_method ) { method = generator_method; Init(); };
	double generate();

};

#endif /* RANDOMGENERATOR_H_ */
