/*
 * Source.h
 *
 *  Created on: Aug 7, 2011
 *      Author: janez
 */

#ifndef SOURCE_H_
#define SOURCE_H_

#include "peg_tc.h"
#include "Species.h"
#include "Vector.h"
#include "ParticleList.h"
#include "SpatialRegion.h"
#include "Compute_green.h"

#define TYPE_DELTA_UNDEFINED	0
#define TYPE_DELTA_NONE			1		// for use as initLoad
#define TYPE_DELTA_TIME			2
#define TYPE_DELTA_STEP			3

class Source {
protected:
	int type, position_dist, velocity_dist;
	int sp_index;
	Species sp;
	Vector3 inject_r, inject_v, inject_r_min, inject_r_max, inject_v_min, inject_v_max;
	Vector3 inject_T;
	long int numPartToInject, inject_dStep;
	double inject_dt, simulation_dt;
	double lastInjectTime;
	long int lastInjectStep;
	std::vector<double> v_distribution;

	int isSpeciesDefined(std::string name, std::vector<Species> &species);
	void dumpPositions(std::string offset);
	void dumpVelocities(std::string offset);

	Vector3 inject_velocity(int pi);
	Vector3 inject_position(int pi);
	ParticleList doInjectParticles( ParticleList& plist, Tree &in_tree, SpatialRegion& in_region, double t, long int step ) ;
	void prepareMaxwellianDistributionFactors(double vmin, double vmax, int size, double v_off, bool force);

	inline double fmaxw( double v ) {
		return (exp(-v*v*0.5));
	}

	inline double fvmaxw( double v, double v0 ) {
		double u = v-v0;
		return (v*exp(-u*u*0.5));
	}

public:
	Source();
	virtual ~Source();
	void init();
	void dump(int level, int index);
	void prepare();
	std::vector<std::string> definedDataOK(std::vector<Species> &species);
	long long memory_footprint();

	inline void set_sp( Species in_sp, int index ) { sp = in_sp; sp_index = index; };
	inline void set_r( Vector3 in_r ) { inject_r = in_r; };
	inline void set_r_min( Vector3 in_r ) { inject_r_min = in_r; };
	inline void set_r_max( Vector3 in_r ) { inject_r_max = in_r; };
	inline void set_v( Vector3 in_v ) { inject_v = in_v; };
	inline void set_v_min( Vector3 in_v ) { inject_v_min = in_v; };
	inline void set_v_max( Vector3 in_v ) { inject_v_max = in_v; };
	inline void set_numberToInject( long int in_n ) { numPartToInject = in_n; };
	inline void set_dt( double in_dt ) { inject_dt = in_dt; type = TYPE_DELTA_TIME; };
	inline void set_simulation_dt( double in_dt ) { simulation_dt = in_dt; };
	inline void set_dStep( long int in_n ) { inject_dStep = in_n; type = TYPE_DELTA_STEP; };
	inline void set_position_distribution( int type ) { position_dist = type; };
	inline void set_velocity_distribution( int type ) { velocity_dist = type; };
	inline void set_T( Vector3 in_T ) { inject_T = in_T; };
	inline void set_Tpar( double in_T ) { inject_T.set_e1(in_T); };
	inline void set_Tperp( double in_T ) { inject_T.set_e2(in_T); inject_T.set_e3(in_T); };

	inline Species get_sp() { return sp; };
	inline Vector3 get_r() { return inject_r; };
	inline Vector3 get_r_min() { return inject_r_min; };
	inline Vector3 get_r_max() { return inject_r_max; };
	inline Vector3 get_v() { return inject_v; };
	inline Vector3 get_v_min() { return inject_v_min; };
	inline Vector3 get_v_max() { return inject_v_max; };
	inline long int get_dStep() { return inject_dStep; };
	inline long int get_numberToInject() { return numPartToInject; };
	inline double get_dt() { return inject_dt; };
	inline double get_simulation_dt() { return simulation_dt; };
	inline int get_position_distribution() { return position_dist; };
	inline int get_velocity_distribution() { return velocity_dist; };
	inline Vector3 get_T() { return inject_T; };
	inline double get_Tpar() { return inject_T.e1(); };
	inline double get_Tperp() { return inject_T.e2(); };

	ParticleList inject( ParticleList& plist, Tree &in_tree, SpatialRegion& in_region, int in_sp_index, double t, long int step );
};

#endif /* SOURCE_H_ */
