/**
 * Tree.h
 *
 *  Created on: Aug 22, 2011
 *      Author: janez
 */

#ifndef TREE_H_
#define TREE_H_

#include "peg_tc.h"
#include "ParticleList.h"
#include "SpatialRegion.h"
#include "TreeNode.h"

#define QC_OK				1
#define XC_OK				2
#define NODE_COMPUTED		3

class SpatialRegion;

class Tree {
	unsigned int maxParticlesInNode;
	double node2SystemLengthRatio;
	double p2cRatio;
	TreeNode* root;
	long int maxLevel, numNodes;
	bool treecode_disabled;		// computing via treecode (= false) or direct summation (= true)

	TreeNode *buildTree( ParticleList& parts, SpatialRegion& region, double ds, double de, std::vector<int>& parent_parts, long level, long *maxLevel, long *numNodes );
	void shrinkBox(ParticleList& parts, TreeNode *tree, double *ds, double *de);

public:
	Tree();
	virtual ~Tree();
	void init();
	void dump(int level);
	inline TreeNode* get_root() { return root; };

	inline void set_maxParticlesInNode( int in_n ) { maxParticlesInNode = in_n; };
	inline void set_nodeLengthRatio( double in_ratio ) { node2SystemLengthRatio = in_ratio; };
	inline void set_p2c( double in_ratio ) { p2cRatio = in_ratio; };
	inline void treecode_disable() { treecode_disabled = true; };
	inline void treecode_enable() { treecode_disabled = false; };


	inline int get_maxParticlesInNode() { return maxParticlesInNode; };
	inline double get_nodeLengthRatio() { return node2SystemLengthRatio; };
	inline double get_p2c() { return p2cRatio; };
	inline bool get_treecode_disable() { return treecode_disabled; };

	void build( ParticleList& plist, SpatialRegion& region );
	long long memory_footprint();
	long count_particles();

	inline void freeRoot() { delete root; };
};

#endif /* TREE_H_ */
