/**
 * Tree.h
 *
 *  Created on: Aug 22, 2011
 *      Author: janez
 */

#ifndef TREENODE_H_
#define TREENODE_H_

#include "peg_tc.h"
#include "ParticleList.h"

#define QC_OK				1
#define XC_OK				2
#define NODE_COMPUTED		3

class TreeNode {
	std::vector<unsigned long> particle_index; // indexes of particles in current node
	TreeNode *l1, *l2;		// pointers to both leafs
	double ds, de;					// start and end of domain
	double Qc, Xc;
	// some variables for checking tree generation
	long index, level;

public:
	TreeNode();
	TreeNode(unsigned long noParticles);
	TreeNode(std::vector<int> indexes);
	virtual ~TreeNode();
	void init(unsigned long noParticles);
	void dump(int level);

	inline void addParticleIndex(unsigned long inIndex) { this->particle_index.push_back(inIndex); };
	void addParticleIndexs(std::vector<int> inIndexes);
	inline void setNumberOfParticles(unsigned long noParticles) { particle_index.reserve(noParticles); };
	inline void clearParticleIndexes() { this->particle_index.clear(); };
	inline void setQc( double inQc ) { this->Qc = inQc; };
	inline void setXc( double inXc ) { this->Xc = inXc; };
	inline void setLevel( int inLevel ) { this->level = inLevel; };
	inline void setIndex( int inIndex ) { this->index = inIndex; };
	inline void setDs( double inDs ) { this->ds = inDs; };
	inline void setDe( double inDe ) { this->de = inDe; };
	inline void setLeaf1( TreeNode *node ) { this->l1 = node; } ;
	inline void setLeaf2( TreeNode *node ) { this->l2 = node; } ;

	inline unsigned long getParticleIndex( unsigned long inIndex ) { return (inIndex < this->particle_index.size()) ? this->particle_index[inIndex] : 0; };
	inline unsigned long getNumberOfParticles() { return particle_index.size(); };
	inline double getQc() { return this->Qc; };
	inline double getXc() { return this->Xc; };
	inline int getLevel() { return this->level; };
	inline int getIndex() { return this->index; };
	inline double getDs() { return this->ds; };
	inline double getDe() { return this->de; };
	inline TreeNode *getLeaf1() { return this->l1; } ;
	inline TreeNode *getLeaf2() { return this->l2; } ;

	double getDistanceFromCenter(double x) { return x - this->Xc; };
	double getRadius() { return 0.5*fabs(this->de - this->ds); };

	long long memory_footprint();
	long count_particles();

	void computeQc(ParticleList *pl);
	void computeXc();

};

#endif /* TREENODE_H_ */
